package com.suntray.acl.config;

import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.core.io.ClassPathResource;

import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;

@ConfigurationProperties("rsa.key")
@Data
@Slf4j
public class RsaKeyProperties {
    //证书文件路径
    private String key_location;
    //秘钥库密码
    private String key_password;
    //秘钥密码
    private String keypwd;
    //秘钥别名
    private String alias;

    private PrivateKey privateKey;

    private PublicKey publicKey;

    /**
     * 获取证书中的公钥
     *
     * @return
     * @throws Exception
     */
    public PublicKey getPublicKey() {
        //访问证书路径
        ClassPathResource resource = new ClassPathResource(key_location);
        //创建秘钥工厂
        try {
            KeyStore keyStore = KeyStore.getInstance("JKS");
            keyStore.load(resource.getInputStream(), key_password.toCharArray());
            return keyStore.getCertificate(alias).getPublicKey();
        } catch (Exception e) {
            log.error("获取公钥错误" + e.getStackTrace() + e.getMessage());
            return null;
        }
    }

    /**
     * 获取证书中的私钥
     *
     * @return
     * @throws Exception
     */
    public PrivateKey getPrivateKey() {
        //访问证书路径
        ClassPathResource resource = new ClassPathResource(key_location);
        //创建秘钥工厂
        try {
            KeyStore keyStore = KeyStore.getInstance("JKS");
            keyStore.load(resource.getInputStream(), key_password.toCharArray());
            return (PrivateKey) keyStore.getKey(alias, keypwd.toCharArray());
        } catch (Exception e) {
            log.error("获取私钥错误" + e.getStackTrace() + e.getMessage());
            return null;
        }
    }

}
