package com.suntray.acl.config;

import com.suntray.acl.filter.JwtLoginFilter;
import com.suntray.acl.filter.JwtVerifyFilter;
import com.suntray.acl.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

@Configuration
@EnableWebSecurity
@EnableGlobalMethodSecurity(securedEnabled = true)
public class WebSecurityConfig extends WebSecurityConfigurerAdapter {

    @Autowired
    private UserService userService;

    @Autowired
    private RsaKeyProperties prop;

    //配置用户密码加密方式
    @Bean
    public BCryptPasswordEncoder passwordEncoder() {
        return new BCryptPasswordEncoder();
    }

    //指定认证对象的来源
    public void configure(AuthenticationManagerBuilder auth) throws Exception {
        auth.userDetailsService(userService).passwordEncoder(passwordEncoder());
    }

    //SpringSecurity配置信息
    public void configure(HttpSecurity http) throws Exception {
        //所有请求必须认证通过
        http.csrf().disable() //禁用csrf
                // 配置放行地址
                .authorizeRequests().antMatchers("/ics/login", "/ics/user/add").permitAll()
                .and().authorizeRequests().antMatchers("/**/*swagger*/**",
                "/v2/api-docs",
                "/**/*.js",
                "/**/*.ico",
                "/**/*.css",
                "/socket/**",
                "/**/*.html",
                "/**/sync/**",
                "/**/*.htm",
                "/static/**").permitAll()
                //其他路径需要验证
                .anyRequest().authenticated().and()
                //添加登录和鉴权过滤器
                .addFilter(new JwtLoginFilter(super.authenticationManager(), prop))
                .addFilter(new JwtVerifyFilter(super.authenticationManager(), prop))
                //禁用session
                .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS);
    }

    //配置静态资源放行
    /*@Override
    public void configure(WebSecurity web) throws Exception {
        web.ignoring().antMatchers("/ics/login","/ics/logout",
                "/*.html","/css/**","/data/**","/fonts/**","/img/**","/js/**");
    }*/
}
