package com.suntray.storage.config;


import com.fasterxml.jackson.annotation.JsonFormat;
import com.suntray.annotation.ExcelField;
import com.suntray.utils.ListUtil;
import com.suntray.utils.PageParam;
import com.suntray.utils.PageUtils;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.Getter;
import lombok.NoArgsConstructor;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.web.servlet.view.document.AbstractXlsxView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Member;
import java.lang.reflect.Method;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Excel视图
 *
 */
@Getter
public class ExcelView extends AbstractXlsxView {
    private static final String DATE_FMT = "yyyy-MM-dd HH:mm:ss";

    private String filename;

    private List<ExcelSheet> sheets;


    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class ExcelSheet {
        private String sheetName;
        private String topic;
        private String[] properties;
        private String[] titles;
        private Map<String, Member> propMap;
        private List<?> data;
        private String[] contents;

        public ExcelSheet(String sheetName, String topic, Class<?> model, List<?> data) {
            this.sheetName = sheetName;
            this.data = data;
            this.topic = topic;
            fillTitles(model, this);
        }

        public ExcelSheet(String sheetName, Class<?> model, List<?> data) {
            this(sheetName, null, model, data);
        }
    }

    public static ExcelSheet createSheet(String sheetName, String topic, Class<?> model, List<?> data) {
        return new ExcelSheet(sheetName, topic, model, data);
    }

    public static ExcelSheet createSheet(String sheetName, Class<?> model, List<?> data) {
        return createSheet(sheetName, null, model, data);
    }

    public ExcelView(String filename, String sheetName, String topic, String[] properties, String[] titles, List<?> data, String[] contents) {
        this.filename = filename;
        ExcelSheet sheet = new ExcelSheet();
        sheet.setSheetName("内部");
        sheet.setProperties(properties);
        sheet.setTitles(titles);
        sheet.setData(data);
        sheet.setTopic(topic);
        sheet.setContents(contents);
        sheets = Collections.singletonList(sheet);
    }

    public ExcelView(String filename, String sheetName, String[] properties, String[] titles, List<?> data, String[] contents) {
        this(filename, sheetName, null, properties, titles, data, contents);
    }

    public ExcelView(String[] properties, String[] titles, List<?> data, String[] contents) {
        this(null, null, properties, titles, data, contents);
    }

    public ExcelView(String[] properties, String[] titles, List<?> data) {
        this(properties, titles, data, null);
    }

    public ExcelView(String[] properties, List<?> data) {
        this(properties, null, data);
    }

    public ExcelView(String filename, String sheetName, String topic, Class<?> model, List<?> data, String[] contents) {
        this(filename, sheetName, topic, null, null, data, contents);
        ExcelSheet sheet = sheets.get(0);
        fillTitles(model, sheet);
    }
    //size 属性为多少条数据分一个sheetduosha页
    public ExcelView(String filename, String sheetName, String topic, Class<?> model, List<?> datas, String[] contents, int size) {
        sheets=new ArrayList<>();
        this.filename = filename;
        int sheetSize=0;
        List<? extends List<?>> lists= ListUtil.partition(datas,size);
        datas=null;
        for(List<?> data:lists){
            ExcelSheet sheet = new ExcelSheet();
            sheet.setSheetName("内部");
            sheet.setData(data);
            sheet.setTopic(topic);
            sheet.setContents(contents);
            sheets.add(sheet);
            sheet = sheets.get(sheetSize);
            ++sheetSize;
            fillTitles(model, sheet);
        }
    }

    public ExcelView(String filename, String sheetName, Class<?> model, List<?> data, String[] contents) {
        this(filename, sheetName, null, model,  data, contents);
    }

    public ExcelView(String filename, Class<?> model, String topic, List<?> data) {
        this(filename.replaceAll(".xlsx","（内部）.xlsx"), null, topic+"（内部）", model, data, null);
    }

    public ExcelView(String filename, Class<?> model, String topic, List<?> data, int size) {
        this(filename.replaceAll(".xlsx","（内部）.xlsx"), null, topic+"（内部）", model, data, null,size);
    }

    public ExcelView(String filename, Class<?> model, List<?> data) {
        this(filename.replaceAll(".xlsx","（内部）.xlsx"), model, null, data);
    }

    public ExcelView(String filename, ExcelSheet... sheets) {
        this.filename = filename.replaceAll(".xlsx","（内部）.xlsx");
        this.sheets = new ArrayList<>();
        for (ExcelSheet sheet : sheets) {
            this.sheets.add(sheet);
        }
    }

    /**
     * 从注解中获取导出字段名
     *
     * @param model
     */
    private static void fillTitles(Class<?> model, ExcelSheet sheet) {
        List<ExportField> props = new ArrayList<>();
        Field[] fields = model.getDeclaredFields();

        Map<String,Member> fieldMap = new HashMap<>();
        //从类属性上找注解
        for (Field f : fields) {
            if (f.isAnnotationPresent(ExcelField.class)) {
                ExcelField fd = f.getAnnotation(ExcelField.class);
                ExportField exportField = new ExportField();
                String name = fd.remark();

                //未传备注，从swagger上获取
                if (StringUtils.isEmpty(name)) {
                    ApiModelProperty apiAnn = f.getAnnotation(ApiModelProperty.class);
                    if (apiAnn != null && StringUtils.isNotEmpty(apiAnn.value())) {
                        name = apiAnn.value();
                    } else {
                        name = f.getName();
                    }
                }
                exportField.setName(name);
                exportField.setProperty(f.getName());
                exportField.setOrder(fd.order());
                props.add(exportField);
                fieldMap.put(f.getName(),f);
            }
        }

        //从类方法上找注解
        Method[] methods = model.getDeclaredMethods();
        for (Method m : methods) {
            if (m.isAnnotationPresent(ExcelField.class)) {
                ExcelField fd = m.getAnnotation(ExcelField.class);
                ExportField exportField = new ExportField();

                //把方法名转成属性名
                String fname = StringUtils.uncapitalize(m.getName().replaceFirst("get", ""));
                exportField.setProperty(fname);

                String name = fd.remark();

                //未传备注，从swagger上获取
                if (StringUtils.isEmpty(name)) {
                    ApiModelProperty apiAnn = m.getAnnotation(ApiModelProperty.class);
                    if (apiAnn != null && StringUtils.isNotEmpty(apiAnn.value())) {
                        name = apiAnn.value();
                    } else {
                        name = fname;
                    }
                }
                exportField.setName(name);

                exportField.setOrder(fd.order());
                props.add(exportField);
                fieldMap.put(fname,m);
            }
        }
        List<String> ps = new ArrayList<>(props.size());
        List<String> ts = new ArrayList<>(props.size());
        props.stream().sorted((o1, o2) -> o1.getOrder() - o2.getOrder()).forEach(f -> {
            ps.add(f.getProperty());
            ts.add(f.getName());
        });
        sheet.setProperties(ps.toArray(new String[0]));
        sheet.setTitles(ts.toArray(new String[0]));
        sheet.setPropMap(fieldMap);
    }


    @Override
    protected void buildExcelDocument(Map<String, Object> map, Workbook workbook, HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        int ps = 60000;
        for (ExcelSheet sheet : sheets) {
            List<?> orgData = sheet.getData();
            if(orgData.size()<= ps) {
                buildExcel(workbook, sheet);
            }else{
                //超过60000的拆分多个sheet
                double total = Math.ceil(orgData.size()*1.0 / ps);
                for(int p=1;p<=total;p++) {
                    ExcelSheet split = new ExcelSheet();
                    BeanUtils.copyProperties(split,sheet);
                    List<?> data = PageUtils.page(orgData, new PageParam(p, ps)).getRecords();
                    split.setData(data);
                    split.setSheetName("sheet"+p+"内部");
                    buildExcel(workbook, split);
                }
            }
        }

        //设置响应为文件下载
        httpServletResponse.setContentType("application/force-download");

        boolean firefox = httpServletRequest.getHeader("user-agent").toLowerCase().indexOf("firefox")>-1;
        if (StringUtils.isNotEmpty(filename)) {
            httpServletResponse.setHeader("Content-disposition", "attachment;filename"+(firefox?"*=":"=") + URLEncoder.encode(filename, "UTF-8"));
        } else {
            httpServletResponse.setHeader("Content-disposition", "attachment");
        }
    }

    private void buildExcel(Workbook workbook, ExcelSheet param) throws IllegalAccessException, InvocationTargetException, NoSuchMethodException {
        Sheet sheet;
        if (StringUtils.isNotEmpty(param.getSheetName())) {
            sheet = workbook.createSheet(param.getSheetName());
        } else {
            sheet = workbook.createSheet();
        }

        //存单元格数据最大长度
        Map<Integer, Integer> sizeMap = new HashMap<>();

        int rowNumber = 0;


        //写主题
        if (param.getTopic() != null && param.getTitles() != null && param.getTitles().length > 0) {
            Row top = sheet.createRow(rowNumber);
            top.setHeight((short) 800);
            Cell cell = top.createCell(0);

            CellStyle cellStyle = workbook.createCellStyle();
            cellStyle.setAlignment(HorizontalAlignment.CENTER);
            cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
            cellStyle.setWrapText(true);
            Font font = workbook.createFont();
            font.setFontHeightInPoints((short) 12);
            font.setBold(true);
            cellStyle.setFont(font);
            cell.setCellStyle(cellStyle);
            cell.setCellValue(param.getTopic());

            CellRangeAddress region = new CellRangeAddress(0, 0, 0, param.getTitles().length - 1);
            sheet.addMergedRegion(region);
            rowNumber++;
        }

        //写表头
        if (param.getTitles() != null && param.getTitles().length > 0) {
            Row header = sheet.createRow(rowNumber);
            header.setHeight((short) 400);
            CellStyle cellStyle = workbook.createCellStyle();
            cellStyle.setBorderBottom(BorderStyle.THIN);
            cellStyle.setBorderTop(BorderStyle.THIN);
            cellStyle.setBorderLeft(BorderStyle.THIN);
            cellStyle.setBorderRight(BorderStyle.THIN);
            Font font = workbook.createFont();
            font.setFontHeightInPoints((short) 11);
            font.setBold(true);
            cellStyle.setFont(font);

            cellStyle.setAlignment(HorizontalAlignment.CENTER);
            // 设置垂直对齐的样式为居中对齐;
            cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
            cellStyle.setWrapText(true);
            for (int i = 0; i < param.getProperties().length; i++) {
                Cell cell = header.createCell(i);
                cell.setCellStyle(cellStyle);
                if (param.getTitles().length > i && param.getTitles()[i] != null) {
                    cell.setCellValue(param.getTitles()[i]);
                } else {
                    cell.setCellValue(param.getProperties()[i]);
                }
                sizeMap.put(i, cell.getStringCellValue().getBytes().length);
            }
            rowNumber++;
        }

        //填数据
        if (param.getData() != null) {
            CellStyle cellStyle = workbook.createCellStyle();
            cellStyle.setBorderBottom(BorderStyle.THIN);
            cellStyle.setBorderTop(BorderStyle.THIN);
            cellStyle.setBorderLeft(BorderStyle.THIN);
            cellStyle.setBorderRight(BorderStyle.THIN);
            cellStyle.setAlignment(HorizontalAlignment.CENTER);
            // 设置垂直对齐的样式为居中对齐;
            cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
            cellStyle.setWrapText(true);
            int num = 1;
            Optional<?> first = param.getData().stream().findFirst();
            Field autoField = null;
            if(first.isPresent()){
                autoField = getAutoValueField(first.get());
            }

            for (Object item : param.getData()) {
                Row row = sheet.createRow(rowNumber);
                for (int i = 0; i < param.getProperties().length; i++) {
                    Cell cell = row.createCell(i);
                    cell.setCellStyle(cellStyle);

                    if(autoField!=null&&autoField.getName().equals(param.getProperties()[i])) {
                        autoField.set(item, num);
                    }
                    String val = getValue(param.getPropMap(),item,param.getProperties()[i]);
                    cell.setCellValue(val);

                    //用当前的长度与上一个对比取大的
                    Integer colWidth = sizeMap.get(i);
                    int currWidth = cell.getStringCellValue().getBytes().length;
                    if (colWidth == null || colWidth < currWidth) {
                        sizeMap.put(i, currWidth);
                    }
                }
                rowNumber++;
                num++;
            }

            //设置单元格宽度
            for (int i = 0; i < param.getProperties().length; i++) {
                int w = (2+sizeMap.get(i)) * 256;
                sheet.setColumnWidth(i, w>255*255?6000:w);
            }
        }

        //填自定义的数据
        if (param.getContents() != null && param.getContents().length > 0) {
            rowNumber++;
            CellStyle cellStyle = workbook.createCellStyle();
            cellStyle.setBorderBottom(BorderStyle.THIN);
            cellStyle.setBorderTop(BorderStyle.THIN);
            cellStyle.setBorderLeft(BorderStyle.THIN);
            cellStyle.setBorderRight(BorderStyle.THIN);
            Font font = workbook.createFont();
            font.setColor(HSSFColor.HSSFColorPredefined.GREY_50_PERCENT.getIndex());
            cellStyle.setFont(font);
            for (String content : param.getContents()) {
                Row row = sheet.createRow(rowNumber);
                Cell cell = row.createCell(0);
                cell.setCellStyle(cellStyle);
                cell.setCellValue(content);
                rowNumber++;
            }
        }
    }

    private String getValue(Map<String, Member> propMap, Object item, String property) throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        Class clazz = PropertyUtils.getPropertyType(item, property);
        if(!clazz.equals(Date.class)) {
            return BeanUtils.getProperty(item, property);
        }
        Member member = propMap.get(property);
        String fmt = DATE_FMT;
        Date dateVal = null;
        if(member instanceof Field){
            Field f = (Field) member;
            f.setAccessible(true);
            dateVal = (Date)f.get(item);
            if(f.isAnnotationPresent(JsonFormat.class)){
                fmt = f.getAnnotation(JsonFormat.class).pattern();
            }
        }else if(member instanceof Method){
            Method m = (Method) member;
            dateVal = (Date)m.invoke(item);
            if(m.isAnnotationPresent(JsonFormat.class)){
                fmt = m.getAnnotation(JsonFormat.class).pattern();
            }
        }
        if(dateVal==null){
            return null;
        }
        return new SimpleDateFormat(fmt).format(dateVal);
    }

    private Field getAutoValueField(Object item) {
        Field[] fields = item.getClass().getDeclaredFields();
        for(Field f:fields){
            if(f.isAnnotationPresent(ExcelField.class)){
                ExcelField ann = f.getAnnotation(ExcelField.class);
                if(ann.auto()) {
                    f.setAccessible(true);
                    return f;
                }
            }
        }
        return null;
    }

}
