package com.suntray.storage.config;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.api.IErrorCode;
import com.baomidou.mybatisplus.extension.exceptions.ApiException;
import com.suntray.utils.Result;
import org.mybatis.spring.MyBatisSystemException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.io.EOFException;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipException;

/**
 * 1、全局异常处理
 * 2、表单校验
 */
@RestControllerAdvice
public class GlobalExceptionHandler {
    private static Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    @ExceptionHandler(value = {Exception.class})
    public Result<Object> handleException(Exception e) {

        //业务接口异常
        if (e instanceof ApiException) {
            logger.info("业务异常", e.getMessage());
            ApiException apierr = (ApiException) e;
            IErrorCode errorCode = apierr.getErrorCode();
            return Result.failed(errorCode != null ? errorCode.getMsg() : apierr.getMessage());
        }

        //参数校验
        if (e instanceof MethodArgumentNotValidException) {
            MethodArgumentNotValidException bindException = (MethodArgumentNotValidException) e;
            BindingResult bindingResult = bindException.getBindingResult();
            return getErrors(bindingResult);

        }
        //参数校验
        if (e instanceof BindException) {
            BindException bindException = (BindException) e;
            BindingResult bindingResult = bindException.getBindingResult();
            return getErrors(bindingResult);

        }
        if (e instanceof MissingServletRequestParameterException) {
            MissingServletRequestParameterException me = (MissingServletRequestParameterException) e;
            return Result.failed(me.getParameterName() + "不能为空");
        }
        if (e instanceof HttpMessageNotReadableException) {
            String err = "传参错误";
            logger.error(err, e);
            return Result.failed(err);
        }
        //空指针
        if (e instanceof NullPointerException) {
            return Result.failed("数据未找到");
        }
        //数组下标越界
        if (e instanceof ArrayIndexOutOfBoundsException) {
            return Result.failed("数据异常");
        }
        //类型转换异常
        if (e instanceof NumberFormatException) {
            return Result.failed("数据类型转换异常");
        }
        //执行sql异常
        if (e instanceof MyBatisSystemException) {
            return Result.failed("数据异常");
        }
        //文件已结束异常
        if (e instanceof EOFException) {
            return Result.failed("文件异常结束");
        }
        //文件未找到异常
        if (e instanceof FileNotFoundException) {
            return Result.failed("文件不存在");
        }
        //zip文件读取异常
        if (e instanceof ZipException) {
            return Result.failed("Zip包文件读取异常");
        }
        //使用未知方法
        if (e instanceof NoSuchMethodException) {
            return Result.failed("程序异常");
        }
        String err = "系统内部错误,请联系管理员";
        logger.error(err, e);
        return Result.failed(err);

    }


    private Result<Object> getErrors(BindingResult bindingResult) {
        List<Object> ret = new ArrayList<>();
        if (bindingResult != null && bindingResult.hasErrors()) {
            bindingResult.getFieldErrors().stream().forEach(fe -> {
                Map<String, Object> obj = new HashMap<>();
                obj.put("name", fe.getField());
                obj.put("msg", fe.getDefaultMessage());
                ret.add(obj);
            });

        }
        return Result.failed("参数错误", ret);
    }
}
