package com.suntray.storage.config;

import com.baomidou.mybatisplus.extension.exceptions.ApiException;
import com.suntray.annotation.EnableLock;
import com.suntray.storage.lock.DbLock;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.DefaultParameterNameDiscoverer;
import org.springframework.expression.Expression;
import org.springframework.expression.common.TemplateParserContext;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.util.concurrent.locks.Lock;


/**
 * 锁拦截器
 */
@Aspect
@Component
public class LockAspect {

    private Logger logger = LoggerFactory.getLogger(LockAspect.class);

    private SpelExpressionParser parser = new SpelExpressionParser();
    private DefaultParameterNameDiscoverer discoverer = new DefaultParameterNameDiscoverer();

    @Pointcut("@annotation(com.suntray.annotation.EnableLock)")
    public void hasLock() {

    }

    @Around("hasLock()")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        MethodSignature signature = (MethodSignature) point.getSignature();
        Method method = signature.getMethod();
        EnableLock ann = method.getAnnotation(EnableLock.class);

        //获取SpEL表达式的值
        StandardEvaluationContext context = new StandardEvaluationContext();
        Object[] args = point.getArgs();
        String[] parameterNames = discoverer.getParameterNames(method);
        for (int i = 0; i < args.length; i++) {
            context.setVariable(parameterNames[i], args[i]);
        }
        Expression expression = parser.parseExpression(ann.name(),new TemplateParserContext());
        String lockName = expression.getValue(context).toString();
        lockName = point.getTarget().getClass().getSimpleName()+"_"+lockName;
        //加锁
        Lock lock = new DbLock(lockName, ann.holdSeconds(), ann.remark());
        if (lock.tryLock()) {
            try {
                Object ret = point.proceed();
                lock.unlock();
                return ret;
            }catch (Exception e){
                lock.unlock();
                throw e;
            }
        } else {
            throw new ApiException(StringUtils.isEmpty(ann.message()) ? "服务繁忙，请稍候再试" : ann.message());
        }

    }
}
