package com.suntray.storage.config;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.baomidou.mybatisplus.extension.incrementer.OracleKeyGenerator;
import com.baomidou.mybatisplus.extension.plugins.PaginationInterceptor;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.core.context.SecurityContextHolder;

import java.util.Date;

/**
 * mybatis plus配置
 */
@Configuration
public class MybatisConfig {

    /**
     * 分页插件
     */
    @Bean
    public PaginationInterceptor paginationInterceptor() {
        PaginationInterceptor paginationInterceptor = new PaginationInterceptor();
        paginationInterceptor.setLimit(-1);
        paginationInterceptor.setDialectType("mysql");
        return paginationInterceptor;
    }

    /**
     * 自动填充字段配置
     */
    @Bean
    public MetaObjectHandler autoFillHandler() {
        return new MetaObjectHandler() {
            @Override
            public void insertFill(MetaObject metaObject) {
                this.setFieldValByName("createTime", new Date(), metaObject);
                this.setFieldValByName("isdel", 0, metaObject);
                String loginName = SecurityContextHolder.getContext().getAuthentication().getName();
                if (loginName != null) {
                    this.setFieldValByName("creater", loginName, metaObject);
                }
            }

            @Override
            public void updateFill(MetaObject metaObject) {
                this.setFieldValByName("updateTime", new Date(), metaObject);
                String loginName = SecurityContextHolder.getContext().getAuthentication().getName();
                if (loginName != null) {
                    this.setFieldValByName("updater", loginName, metaObject);
                }
            }
        };
    }

}
