package com.suntray.storage.endpoint;

import com.suntray.storage.vo.WsMessage;
import com.suntray.utils.JsonUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 外置tomcat部署要去掉@Component
 */
@ServerEndpoint("/socket/{type}")
@Component
public class SuntraySocket {

    private static Logger log = LoggerFactory.getLogger(SuntraySocket.class);

    private static Map<String, List<Session>> clients = new ConcurrentHashMap<>();

    @OnOpen
    public void onOpen(Session session, @PathParam("type") String type) {
        List<Session> sessions = clients.get(type);
        if (sessions == null) {
            sessions = new ArrayList<>();
            clients.put(type, sessions);
        }
        sessions.add(session);
        log.info("有新连接加入" + type);
    }

    @OnClose
    public void onClose(Session session) {
        for (String key : clients.keySet()) {
            List<Session> sessions = clients.get(key);
            if (sessions == null) {
                continue;
            }
            sessions.remove(session);
        }
        clients.remove(session.getId());
    }

    @OnError
    public void onError(Throwable e) {

    }

    @OnMessage
    public void onMessage(String message) {
        log.info("收到消息：" + message);
    }

    public synchronized static void sendMsg(WsMessage message) {
        try {
            List<Session> sessions = clients.get(message.getType());
            if (sessions == null) {
                return;
            }
            String msg = JsonUtils.toString(message);
            for (Session session : sessions) {
                if (session.isOpen()) {
                    session.getBasicRemote().sendText(msg);
                }
            }
        } catch (Exception e) {
            log.error("发送消息失败", e);
        }

    }

}
