package com.suntray.storage.lock;

import com.suntray.storage.domain.Dblock;
import com.suntray.storage.mapper.DblockMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 检测锁，过期释放
 */
@Component
public class CheckExpire implements Runnable, InitializingBean {
    private static final Logger log = LoggerFactory.getLogger(CheckExpire.class);
    private volatile boolean stop;

    @Autowired
    private DblockMapper dblockMapper;

    @PostConstruct
    public void init(){
        Thread thread = new Thread(this);
        thread.setName("CheckExpire");
        thread.start();
    }


    @PreDestroy
    public void destroy(){
        stop=true;
    }

    @Override
    public void run() {
        while(!stop){
            try {
                for (DbLock lock : DbLock.locks.values()) {
                    if(lock.isExpired()){
                        lock.unlock();
                    }
                }
                TimeUnit.SECONDS.sleep(1);
            }catch (Exception e){
                log.error(e.getMessage());
                //e.printStackTrace()
            }
        }
    }

    @Override
    public void afterPropertiesSet() {
        List<Dblock> locks = dblockMapper.selectList(null);

        //删除过期的锁
        Date now = new Date();
        List<String> ids = locks.stream().filter(l->l.getExpireTime().before(now)).map(Dblock::getRs).collect(Collectors.toList());
        if (!ids.isEmpty()) {
            dblockMapper.deleteBatchIds(ids);
        }

        //初始化未过期的锁
        locks.stream().filter(l->l.getExpireTime().after(now)).forEach(lock->{
            DbLock dbLock = new DbLock(lock.getRs(),null,lock.getRemark());
            Calendar c = Calendar.getInstance();
            c.setTime(lock.getExpireTime());
            dbLock.setExpireTime(c.getTimeInMillis());
            DbLock.locks.put(lock.getRs(),dbLock);
        });
    }
}
