package com.suntray.utils;

import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Supplier;


/**
 * 拷贝工具类
 *
 */
public class BeanCopyUtils {
    public static interface Callback<S, T> {
        void apply(S s, T t);
    }

    public static <S, T> List<T> copyList(List<S> srcList, Supplier<T> target) {
        return copyList(srcList, target, null);
    }

    public static <S, T> List<T> copyList(List<S> srcList, Supplier<T> target, Callback<S, T> callback) {
        List<T> ret = new ArrayList<>();
        if (srcList == null || srcList.isEmpty()) {
            return ret;
        }
        srcList.forEach(s -> {
            T t = target.get();
            BeanUtils.copyProperties(s, t);
            if (callback != null) {
                callback.apply(s, t);
            }
            ret.add(t);
        });
        return ret;
    }
}
