package com.suntray.utils;

import com.baomidou.mybatisplus.extension.exceptions.ApiException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.InetAddress;
import java.net.InterfaceAddress;
import java.net.NetworkInterface;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.stream.Collectors;


/**
 * 网络工具类
 */
public class NetUtils {
    private static final Logger log = LoggerFactory.getLogger(NetUtils.class);
    /**
     * 获取本机mac地址列表
     *
     * @return
     */
    public static List<String> getMacList() {
        try {
            Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces();
            List<String> ret = new ArrayList<>();
            while (en.hasMoreElements()) {
                NetworkInterface iface = en.nextElement();
                List<InterfaceAddress> addrs = iface.getInterfaceAddresses();
                for (InterfaceAddress addr : addrs) {
                    InetAddress ip = addr.getAddress();
                    NetworkInterface network = NetworkInterface.getByInetAddress(ip);
                    if (network == null) {
                        continue;
                    }
                    byte[] mac = network.getHardwareAddress();
                    if (mac == null) {
                        continue;
                    }
                    StringBuilder str = new StringBuilder();
                    for (int i = 0; i < mac.length; i++) {
                        str.append(String.format("%02x%s", mac[i], (i < mac.length - 1) ? "-" : ""));
                    }
                    ret.add(str.toString());
                }

            }
            return ret.stream().distinct().collect(Collectors.toList());
        } catch (Exception e) {
            log.error(e.getMessage());
            //e.printStackTrace()
            throw new ApiException("获取mac出错", e);
        }
    }

    public static String getMacAddr() {
        List<String> macList = getMacList();
        return macList.isEmpty() ? null : macList.get(0);
    }

}
