package com.suntray.utils;

import com.baomidou.mybatisplus.extension.exceptions.ApiException;
import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 汉字转拼音工具类
 *
 */
public class PinyinUtils {

    /**
     * 获取汉字拼音，包括简拼，全拼，多音字用,分隔
     *
     * @param name
     * @return
     */
    public static String getPinyinByComma(String name) {
        List<String> simble = getPinyin(name, false);
        List<String> full = getPinyin(name, true);
        List<String> list = new ArrayList<>(simble);
        list.addAll(full);
        return String.join(",", list);

    }

    /**
     * 获取汉字拼音，多音字用,分隔
     *
     * @param name
     * @param full 是否全拼
     * @return
     */
    public static String getPinyinByComma(String name,boolean full) {
        return String.join(",", getPinyin(name,full));

    }

    /**
     * 获取汉字拼音，支持多音字
     *
     * @param name
     * @param full 是否全拼
     * @return
     */
    public static List<String> getPinyin(String name,boolean full) {
        try {
            HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
            defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
            List<List<String>> list = new ArrayList<>();
            for (int i = 0; i < name.length(); i++) {
                char c = name.charAt(i);
                String[] arr = PinyinHelper.toHanyuPinyinStringArray(c, defaultFormat);
                if (arr == null) {
                    continue;
                }
                List<String> chars = Stream.of(arr).map(s -> full?s:String.valueOf(s.charAt(0))).distinct().collect(Collectors.toList());
                list.add(chars);
            }
            List<String> result = new ArrayList<>();
            pinyin(list, 0, result, new StringBuilder());
            return result;
        } catch (Exception e) {
            throw new ApiException("转拼音出错", e);
        }

    }

    private static void pinyin(List<List<String>> data, int start, List<String> result, StringBuilder cache) {
        List<String> curr = data.get(start);
        for (int i = 0; i < curr.size(); i++) {
            //保存每一个循环的数据，用上级循环的数据+当前循环的数据
            StringBuilder item = new StringBuilder(cache.toString());
            item.append(curr.get(i));
            if (start == data.size() - 1) {
                //到最后一个循环了就往结果List里添加
                result.add(item.toString());
            } else {
                //递归
                pinyin(data, start + 1, result, item);
            }
        }
    }
}
