package com.suntray.utils;

import com.baomidou.mybatisplus.extension.exceptions.ApiException;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @Description:自定义返回结果
 */
@Data
@ApiModel("返回数据对象")
public class Result<T>{
    private Map<String,Object> atts = new HashMap<>();

    @ApiModelProperty("状态码")
    private long code;//状态码
    @ApiModelProperty("错误信息")
    private String msg;//错误码

    @ApiModelProperty("业务数据")
    private T data;//相应具体的数据类型
    public static final long success = 0;
    public static final long failure = -1;

    private static final String success_msg = "操作成功";
    private static final String failure_msg = "操作失败";

    public long getCode() {
        return code;
    }

    public boolean isSuccess() {
        return this.code == success;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public String getMsg() {
        return msg;
    }

    public void setMsg(String msg) {
        this.msg = msg;
    }

    public T getData() {
        return data;
    }

    public void setData(T data) {
        this.data = data;
    }

    public void setAttr(String key,Object val){
        atts.put(key,val);
    }

    public Object getAttr(String key){
        return atts.get(key);
    }

    public Result(long code, String msg, T data) {
        this.code = code;
        this.msg = msg;
        this.data = data;
    }

    public Result(long code, String msg) {
        this.code = code;
        this.msg = msg;
    }


    public static Result ok() {
        return new Result(success, success_msg, null);
    }

    public static Result ok(String msg) {
        return new Result(success, msg, null);
    }

    public static <T> Result ok(T data) {
        return new Result<T>(success, success_msg, data);
    }

    public static <T> Result ok(String msg, T data) {
        return new Result<T>(success, msg, data);
    }

    public static <T> Result failed() {
        return new Result<T>(failure, failure_msg, null);
    }

    public static <T> Result error() {
        throw new ApiException(failure_msg);
    }

    public static <T> Result error(String error) {
        throw new ApiException(error != null ? error : failure_msg);
    }

    public static Result failed(String msg) {
        return new Result(failure, msg, null);
    }

    public static <T> Result failed(T data) {
        return new Result<T>(failure, failure_msg, data);
    }

    public static <T> Result failed(String msg, T data) {
        return new Result<T>(failure, msg, data);
    }

}
